import abc
import json
import sqlite3
import uuid


class Beaker(abc.ABC):
    def __init__(self, name: str, recipe):
        self.name = name
        self.recipe = recipe

    def __repr__(self):
        return f"Beaker({self.name})"

    @abc.abstractmethod
    def items(self):
        pass

    @abc.abstractmethod
    def __len__(self):
        pass

    @abc.abstractmethod
    def add_item(self, item: dict, id: str | None = None) -> None:
        pass

    @abc.abstractmethod
    def reset(self):
        pass


class TempBeaker(Beaker):
    def __init__(self, name: str, recipe):
        super().__init__(name, recipe)
        self._items = []

    def __len__(self):
        return len(self._items)

    def add_item(self, item: dict, id=None) -> None:
        self._items.append((id, item))

    def items(self):
        yield from self._items

    def reset(self):
        self._items = []


class SqliteBeaker(Beaker):
    def __init__(self, name: str, recipe):
        super().__init__(name, recipe)
        # create table if it doesn't exist
        self.cursor = self.recipe.db.cursor()
        self.cursor.row_factory = sqlite3.Row
        self.cursor.execute(
            f"CREATE TABLE IF NOT EXISTS {self.name} (uuid TEXT PRIMARY KEY, data JSON)"
        )

    def items(self):
        self.cursor.execute(f"SELECT uuid, data FROM {self.name}")
        data = self.cursor.fetchall()
        for item in data:
            yield item["uuid"], json.loads(item["data"])

    def __len__(self):
        self.cursor.execute(f"SELECT COUNT(*) FROM {self.name}")
        return self.cursor.fetchone()[0]

    def add_item(self, item: dict, id: str | None = None) -> None:
        if id is None:
            id = str(uuid.uuid1())
        self.cursor.execute(
            f"INSERT INTO {self.name} (uuid, data) VALUES (?, ?)",
            (id, json.dumps(item)),
        )
        self.recipe.db.commit()

    def reset(self):
        self.cursor.execute(f"DELETE FROM {self.name}")
        self.recipe.db.commit()
